import { xml2js } from 'xml-js';
import { convertToUnist, findDoi } from './utils';
import { select, selectAll } from 'unist-util-select';
import { Tags } from './types';
export class Jats {
    constructor(data) {
        this.raw = xml2js(data, { compact: false });
        const { declaration, elements } = this.raw;
        this.declaration = declaration === null || declaration === void 0 ? void 0 : declaration.attributes;
        if (!((elements === null || elements === void 0 ? void 0 : elements.length) === 2 && elements[0].type === 'doctype' && elements[1].name === 'article')) {
            throw new Error('article is not the only element of the JATS');
        }
        this.doctype = elements[0].doctype;
        this.tree = convertToUnist(elements[1]);
    }
    get front() {
        return select(Tags.front, this.tree);
    }
    get premissions() {
        return select(Tags.permissions, this.front);
    }
    get doi() {
        return findDoi(this.front);
    }
    get publicationDates() {
        return selectAll(Tags.pubDate, this.front);
    }
    get publicationDate() {
        return this.publicationDates.find((d) => !!select(Tags.day, d));
    }
    get license() {
        return select(Tags.license, this.premissions);
    }
    get keywordGroup() {
        return select(Tags.kwdGroup, this.front);
    }
    /** The first keywords */
    get keywords() {
        return selectAll(Tags.kwd, this.keywordGroup);
    }
    get keywordGroups() {
        return selectAll(Tags.kwdGroup, this.front);
    }
    get titleGroup() {
        return select(Tags.titleGroup, this.front);
    }
    get articleTitle() {
        return select(Tags.articleTitle, this.titleGroup);
    }
    get articleSubtitle() {
        return select(Tags.subtitle, this.titleGroup);
    }
    get abstract() {
        return select(Tags.abstract, this.front);
    }
    get abstracts() {
        return selectAll(Tags.abstract, this.front);
    }
    get contribGroup() {
        return select(Tags.contribGroup, this.front);
    }
    get contribGroups() {
        return selectAll(Tags.contribGroup, this.front);
    }
    get articleAuthors() {
        return selectAll(Tags.contrib, this.contribGroup);
    }
    get body() {
        return select(Tags.body, this.tree);
    }
    get back() {
        return select(Tags.back, this.tree);
    }
    get subArticles() {
        return selectAll(Tags.subArticle, this.tree);
    }
    get refList() {
        return select(Tags.refList, this.back);
    }
    get references() {
        return selectAll(Tags.ref, this.refList);
    }
}
